﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using Tomham.Collections.Generic;

namespace ClipExtension.Common
{
	[Serializable]
	public sealed class DataUnit
	{
		private List<IData> ItemsStore;

		private DateTime LastUsedTimeStore = DateTime.Now;
		private string NameStore;
		private List<string> TagsStore;

		public List<IData> Items
		{
			get
			{
				if ( ItemsStore == null )
				{
					ItemsStore = new List<IData>();
				}

				return ItemsStore;
			}

			set { ItemsStore = value; }
		}

		public string Name
		{
			get
			{
				if ( NameStore == null )
				{
					NameStore = string.Empty;
				}

				return NameStore;
			}
			set { NameStore = value; }
		}

		[XmlIgnore]
		public DateTime LastUsedTime
		{
			get { return LastUsedTimeStore; }
			set { LastUsedTimeStore = value; }
		}

		public List<string> Tags
		{
			get
			{
				if ( TagsStore == null )
				{
					TagsStore = new List<string>();
				}

				return TagsStore;
			}

			set { TagsStore = value; }
		}

		public void UpdateLastUsedTime()
		{
			LastUsedTime = DateTime.Now;
		}

		public DataUnit Clone()
		{
			var result = new DataUnit { Name = Name, LastUsedTime = LastUsedTime };

			result.Tags.AddRange( Tags );

			foreach ( IData item in Items )
			{
				result.Items.Add( item.Clone() );
			}

			return result;
		}

		public override bool Equals( object obj )
		{
			if ( ReferenceEquals( null, obj ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, obj ) )
			{
				return true;
			}
			if ( obj.GetType() !=
			     typeof ( DataUnit ) )
			{
				return false;
			}

			var other = (DataUnit) obj;

			return Name.Equals( other.Name ) && Items.SequenceEqual( other.Items );
		}

		public override string ToString()
		{
			var builder = new StringBuilder();

			if ( Name != string.Empty )
			{
				builder.Append( Name );
				builder.Append( ": " );
				builder.Append( "\r\n" );
				builder.Append( "\r\n" );
			}

			IEnumerable<string> dataInformations = Items.Select( data => data.ToString() );
			builder.Append( string.Join( "\r\n\r\n", dataInformations ) );

			return builder.ToString();
		}

		public override int GetHashCode()
		{
			return Name.GetHashCode() ^ Items.SequenceGetHashCode();
		}
	}
}