﻿using System.Collections.Generic;
using Tomham.Collections.Generic;

namespace ClipExtension.Common
{
	public class Repository
	{
		private const int DefaultMaxHistoryItemCount = 100;
		public const int MaxHistoryItemCountLimit = 1000 * 1000;
		private List<DataUnit> FavoritesStore;
		private List<DataUnit> HistoryStore;
		private int MaxHistoryItemCountStore;

		public Repository()
		{
			MaxHistoryItemCount = DefaultMaxHistoryItemCount;
		}

		public List<DataUnit> History
		{
			get
			{
				if ( HistoryStore == null )
				{
					HistoryStore = new List<DataUnit>();
				}

				return HistoryStore;
			}

			set { HistoryStore = value; }
		}

		public List<DataUnit> Favorites
		{
			get
			{
				if ( FavoritesStore == null )
				{
					FavoritesStore = new List<DataUnit>();
				}

				return FavoritesStore;
			}

			set { FavoritesStore = value; }
		}

		public int MaxHistoryItemCount
		{
			get
			{
				if ( MaxHistoryItemCountStore < 0 )
				{
					MaxHistoryItemCountStore = DefaultMaxHistoryItemCount;
				}
				else if ( MaxHistoryItemCountLimit < MaxHistoryItemCountStore )
				{
					MaxHistoryItemCountStore = MaxHistoryItemCountLimit;
				}

				return MaxHistoryItemCountStore;
			}
			set { MaxHistoryItemCountStore = value; }
		}

		public void PushToHistory( DataUnit dataUnit )
		{
			History.Insert( 0, dataUnit );

			AdjustHistory();
		}

		public void AdjustHistory()
		{
			History.Sort( new DataUnitComparer() );
			History.RemoveExcessItems( MaxHistoryItemCount );
		}

		public void AdjustFavorites()
		{
			Favorites.Sort( new DataUnitComparer() );
		}

		public void Adjust()
		{
			AdjustHistory();
			AdjustFavorites();
		}
	}
}