﻿using System;
using System.Linq;
using ClipExtension.Common;

namespace ClipExtension.Main
{
	[Serializable]
	public class DataUnitSerializationItem
	{
		public DataUnit Target { get; set; }
		public long LastUsedTimeTicks { get; set; }

		public void UpdateToTarget()
		{
			Target.LastUsedTime = new DateTime( LastUsedTimeTicks );
		}

		public void UpdateFromTarget()
		{
			LastUsedTimeTicks = Target.LastUsedTime.Ticks;
		}

		public bool Equals( DataUnitSerializationItem other )
		{
			if ( ReferenceEquals( null, other ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, other ) )
			{
				return true;
			}
			return Equals( other.Target, Target ) &&
			       Target.Tags.SequenceEqual( other.Target.Tags ) &&
			       other.LastUsedTimeTicks == LastUsedTimeTicks;
		}

		public override bool Equals( object obj )
		{
			if ( ReferenceEquals( null, obj ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, obj ) )
			{
				return true;
			}
			if ( obj.GetType() !=
			     typeof ( DataUnitSerializationItem ) )
			{
				return false;
			}
			return Equals( (DataUnitSerializationItem) obj );
		}

		public override int GetHashCode()
		{
			unchecked
			{
				return ( ( Target != null ? Target.GetHashCode() ^ Target.Tags.GetHashCode() : 0 ) * 397 ) ^
				       LastUsedTimeTicks.GetHashCode();
			}
		}
	}
}