﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using ClipExtension.Common;
using ClipExtension.Plugins;
using Tomham;

namespace ClipExtension.Main
{
	public class FormatSupporterUnit
	{
		public FormatSupporterUnit( List<Lazy<IFormatSupporter, IPluginMetadata>> all,
		                            PluginsPreference pluginsPreference,
		                            IMessageOutput output )
		{
			Output = output;
			PluginContainer = new PluginContainer<IFormatSupporter>( all, pluginsPreference );
		}

		public IMessageOutput Output { get; private set; }

		public PluginContainer<IFormatSupporter> PluginContainer { get; private set; }

		public object CreateDataObject( DataUnit dataUnit )
		{
			var composer = new DataObjectComposer( new DataObject() );

			foreach ( IData data in dataUnit.Items )
			{
				var args = new SetUpDataObjectArgs( Output, data, composer );
				SetUpDataObject( args );
			}

			return composer.Compose();
		}

		private void SetUpDataObject( SetUpDataObjectArgs args )
		{
			Lazy<IFormatSupporter, IPluginMetadata> formatSupporter =
				PluginContainer.Items.FirstOrDefault( item => item.Value.Supported( args.Data ) );

			if ( formatSupporter != null )
			{
				try
				{
					formatSupporter.Value.SetUpDataObject( args );
				}
				catch ( Exception e )
				{
					Output.WriteLine( e.Message );
					Output.WriteLine( e.StackTrace );
				}
			}
		}

		public DataUnit CreateDataUnit( IDataObject iDataObject )
		{
			DataUnitComposer dataUnitComposer = CreateComposer( iDataObject );

			return dataUnitComposer.Compose();
		}

		private DataUnitComposer CreateComposer( IDataObject iDataObject )
		{
			var composer = new DataUnitComposer();
			var args = new SetUpDataArgs( Output, iDataObject, composer );

			foreach ( var formatSupporter in PluginContainer.Items )
			{
				if ( formatSupporter.Value.Supported( iDataObject ) )
				{
					try
					{
						formatSupporter.Value.SetUpData( args );
					}
					catch ( Exception e )
					{
						Output.WriteLine( e.Message );
						Output.WriteLine( e.StackTrace );
					}
				}
			}

			return composer;
		}
	}
}