﻿using System;
using System.Collections.Generic;
using System.Linq;
using ClipExtension.Plugins;
using Tomham.Collections.Generic;

namespace ClipExtension.Main
{
	public class PluginContainer<T>
	{
		public PluginContainer( List<Lazy<T, IPluginMetadata>> items, PluginsPreference pluginsPreference )
		{
			Items = new List<Lazy<T, IPluginMetadata>>();
			Items.AddRange( items );

			pluginsPreference.PriorityIDs.RemoveDuplicateItems();

			MoveUpPriorityItems( pluginsPreference );
		}

		public List<Lazy<T, IPluginMetadata>> Items { get; private set; }

		private void MoveUpPriorityItems( PluginsPreference pluginsPreference )
		{
			var temp = new List<Lazy<T, IPluginMetadata>>();

			foreach ( PluginID id in pluginsPreference.PriorityIDs )
			{
				Lazy<T, IPluginMetadata> item = Find( id );

				if ( item != null )
				{
					temp.Add( item );
					Items.Remove( item );
				}
			}

			Items.InsertRange( 0, temp );
		}

		private Lazy<T, IPluginMetadata> Find( PluginID id )
		{
			return Items.FirstOrDefault( item => item.Metadata.Name == id.Name && item.Metadata.Author == id.Author );
		}
	}
}