﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using ClipExtension.Main;
using Tomham.Collections.Generic;
using Tomham.Windows.Forms;

namespace ClipExtension.Options
{
	[Serializable]
	public class CommonOptions
	{
		private PluginsPreference FormatSupportersPluginsPreferenceStore;
		private Dictionary<string, CombinationKeys> GeneralCommandKeysStore;
		private List<Paster> PastersStore;
		private PluginsPreference ProcessorsPluginsPreferenceStore;

		public CommonOptions()
		{
			ShowIncrementalSearchFormKey = new CombinationKeys { KeyCode = Keys.Z, Modifiers = Modifiers.Alt };

			MaxHistoryItemCount = 100;
			CopyReTryDelay = 100;
			CopyReTryTimes = 3;
			BeforePasteDelay = 100;
			CheckAndRegisterChainInterval = 1000 * 10;
		}

		#region Keys

		public CombinationKeys ShowIncrementalSearchFormKey { get; set; }
		public CombinationKeys ShowGeneralCommandSearchFormKey { get; set; }
		public CombinationKeys ShowRepositoryEditFormKey { get; set; }

		#endregion

		public int MaxHistoryItemCount { get; set; }
		public int CopyReTryDelay { get; set; }
		public int CopyReTryTimes { get; set; }
		public int CheckAndRegisterChainInterval { get; set; }
		public bool CheckAndRegisterChainEnabled { get; set; }
		public int BeforePasteDelay { get; set; }

		public Dictionary<string, CombinationKeys> GeneralCommandKeys
		{
			get
			{
				if ( GeneralCommandKeysStore == null )
				{
					GeneralCommandKeysStore = new Dictionary<string, CombinationKeys>();
				}

				return GeneralCommandKeysStore;
			}
			set { GeneralCommandKeysStore = value; }
		}

		public List<Paster> Pasters
		{
			get
			{
				if ( PastersStore == null )
				{
					PastersStore = new List<Paster>();
				}

				return PastersStore;
			}

			set { PastersStore = value; }
		}

		public PluginsPreference FormatSupportersPluginsPreference
		{
			get
			{
				if ( FormatSupportersPluginsPreferenceStore == null )
				{
					FormatSupportersPluginsPreferenceStore = new PluginsPreference();
				}

				return FormatSupportersPluginsPreferenceStore;
			}
			set { FormatSupportersPluginsPreferenceStore = value; }
		}

		public PluginsPreference ProcessorsPluginsPreference
		{
			get
			{
				if ( ProcessorsPluginsPreferenceStore == null )
				{
					ProcessorsPluginsPreferenceStore = new PluginsPreference();
				}

				return ProcessorsPluginsPreferenceStore;
			}
			set { ProcessorsPluginsPreferenceStore = value; }
		}

		public bool Equals( CommonOptions other )
		{
			if ( ReferenceEquals( null, other ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, other ) )
			{
				return true;
			}
			return Equals( other.FormatSupportersPluginsPreference, FormatSupportersPluginsPreference ) &&
			       Pasters.SequenceEqual( other.Pasters ) &&
			       Equals( other.ProcessorsPluginsPreference, ProcessorsPluginsPreference ) &&
			       other.ShowIncrementalSearchFormKey.Equals( ShowIncrementalSearchFormKey ) &&
			       other.ShowGeneralCommandSearchFormKey.Equals( ShowGeneralCommandSearchFormKey ) &&
			       other.ShowRepositoryEditFormKey.Equals( ShowRepositoryEditFormKey ) &&
			       other.MaxHistoryItemCount == MaxHistoryItemCount && other.CopyReTryDelay == CopyReTryDelay &&
			       other.CopyReTryTimes == CopyReTryTimes && other.CheckAndRegisterChainInterval == CheckAndRegisterChainInterval &&
			       other.BeforePasteDelay == BeforePasteDelay &&
			       other.CheckAndRegisterChainEnabled.Equals( CheckAndRegisterChainEnabled ) &&
			       other.GeneralCommandKeys.SequenceEqual( GeneralCommandKeys );
		}

		public override bool Equals( object obj )
		{
			if ( ReferenceEquals( null, obj ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, obj ) )
			{
				return true;
			}
			if ( obj.GetType() !=
			     typeof ( CommonOptions ) )
			{
				return false;
			}
			return Equals( (CommonOptions) obj );
		}

		public override int GetHashCode()
		{
			unchecked
			{
				int result = FormatSupportersPluginsPreference.GetHashCode();
				result = ( result * 397 ) ^ Pasters.SequenceGetHashCode();
				result = ( result * 397 ) ^ ProcessorsPluginsPreference.GetHashCode();
				result = ( result * 397 ) ^ ShowIncrementalSearchFormKey.GetHashCode();
				result = ( result * 397 ) ^ ShowGeneralCommandSearchFormKey.GetHashCode();
				result = ( result * 397 ) ^ ShowRepositoryEditFormKey.GetHashCode();
				result = ( result * 397 ) ^ MaxHistoryItemCount;
				result = ( result * 397 ) ^ CopyReTryDelay;
				result = ( result * 397 ) ^ CopyReTryTimes;
				result = ( result * 397 ) ^ BeforePasteDelay;
				result = ( result * 397 ) ^ CheckAndRegisterChainInterval;
				result = ( result * 397 ) ^ CheckAndRegisterChainEnabled.GetHashCode();
				result = ( result * 397 ) ^ GeneralCommandKeys.SequenceGetHashCode();
				return result;
			}
		}
	}
}