﻿using System;
using System.Drawing;
using System.Windows.Forms;
using ClipExtension.Controls;
using Tomham.Windows.Forms;

namespace ClipExtension.Options
{
	[Serializable]
	public class IncrementalSearchFormOptions
	{
		private const Keys DefaultShowCommandSearchFormKey = Keys.Control | Keys.Shift | Keys.Enter;
		private const Keys DefaultCopyKey = Keys.Control | Keys.Enter;
		private const Keys DefaultCopyAndPasteKey = Keys.Enter;
		private Keys CopyAndPasteKeyStore;
		private Keys CopyKeyStore;

		private Keys ShowCommandSearchFormKeyStore;

		public IncrementalSearchFormOptions()
		{
			BackColor = Color.White;
			ForeColor = Color.Black;
			Font = new Font( "MS UI Gothic", 9 );

			MaxShowItemCount = 50;

			WindowStates = new WindowStates { StartPosition = FormStartPosition.CenterScreen, Size = new Size( 800, 600 ) };
		}

		public bool FirstMatching { get; set; }

		public bool CaseSensitive { get; set; }

		public SearchMode SearchMode { get; set; }

		public int MaxShowItemCount { get; set; }

		public Keys ShowCommandSearchFormKey
		{
			get
			{
				if ( ShowCommandSearchFormKeyStore == Keys.None ||
				     KeysHelper.IsModifiersOnly( ShowCommandSearchFormKeyStore ) )
				{
					ShowCommandSearchFormKeyStore = DefaultShowCommandSearchFormKey;
				}
				return ShowCommandSearchFormKeyStore;
			}
			set { ShowCommandSearchFormKeyStore = value; }
		}

		public Keys CopyKey
		{
			get
			{
				if ( CopyKeyStore == Keys.None ||
				     KeysHelper.IsModifiersOnly( CopyKeyStore ) )
				{
					CopyKeyStore = DefaultCopyKey;
				}
				return CopyKeyStore;
			}
			set { CopyKeyStore = value; }
		}

		public Keys CopyAndPasteKey
		{
			get
			{
				if ( CopyAndPasteKeyStore == Keys.None ||
				     KeysHelper.IsModifiersOnly( CopyAndPasteKeyStore ) )
				{
					CopyAndPasteKeyStore = DefaultCopyAndPasteKey;
				}
				return CopyAndPasteKeyStore;
			}
			set { CopyAndPasteKeyStore = value; }
		}

		#region View

		public Color BackColor { get; set; }
		public Color ForeColor { get; set; }
		public Font Font { get; set; }

		public bool InformationPanelCollapsed { get; set; }
		public WindowStates WindowStates { get; set; }

		#endregion

		public bool Equals( IncrementalSearchFormOptions other )
		{
			if ( ReferenceEquals( null, other ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, other ) )
			{
				return true;
			}
			return Equals( other.CopyAndPasteKey, CopyAndPasteKey ) && Equals( other.CopyKey, CopyKey ) &&
			       Equals( other.ShowCommandSearchFormKey, ShowCommandSearchFormKey ) &&
			       other.FirstMatching.Equals( FirstMatching ) && other.CaseSensitive.Equals( CaseSensitive ) &&
			       Equals( other.SearchMode, SearchMode ) && other.MaxShowItemCount == MaxShowItemCount &&
			       other.BackColor.Equals( BackColor ) && other.ForeColor.Equals( ForeColor ) && Equals( other.Font, Font ) &&
			       other.InformationPanelCollapsed.Equals( InformationPanelCollapsed ) &&
			       Equals( other.WindowStates, WindowStates );
		}

		public override bool Equals( object obj )
		{
			if ( ReferenceEquals( null, obj ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, obj ) )
			{
				return true;
			}
			if ( obj.GetType() !=
			     typeof ( IncrementalSearchFormOptions ) )
			{
				return false;
			}
			return Equals( (IncrementalSearchFormOptions) obj );
		}

		public override int GetHashCode()
		{
			unchecked
			{
				int result = CopyAndPasteKey.GetHashCode();
				result = ( result * 397 ) ^ CopyKey.GetHashCode();
				result = ( result * 397 ) ^ ShowCommandSearchFormKey.GetHashCode();
				result = ( result * 397 ) ^ FirstMatching.GetHashCode();
				result = ( result * 397 ) ^ CaseSensitive.GetHashCode();
				result = ( result * 397 ) ^ SearchMode.GetHashCode();
				result = ( result * 397 ) ^ MaxShowItemCount;
				result = ( result * 397 ) ^ BackColor.GetHashCode();
				result = ( result * 397 ) ^ ForeColor.GetHashCode();
				result = ( result * 397 ) ^ Font.GetHashCode();
				result = ( result * 397 ) ^ InformationPanelCollapsed.GetHashCode();
				result = ( result * 397 ) ^ WindowStates.GetHashCode();
				return result;
			}
		}
	}
}