﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using ClipExtension.Common;
using Tomham.Windows.Forms;
using Tomham.Windows.Forms.AutoGeneration;

namespace ClipExtension
{
	public partial class DataUnitEditForm : Form
	{
		private readonly ControllerCollection ControllerCollection = new ControllerCollection();
		private ControllerFactory ControllerFactory;
		private List<string> Tags;

		public DataUnitEditForm()
		{
			InitializeComponent();
		}

		public DataUnit Target { get; private set; }

		private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
		{
			UpdateToTarget();
			DialogResult = DialogResult.OK;
		}

		public void SetUp( DataUnit target, ControllerFactory factory, List<string> tags )
		{
			Target = target;
			ControllerFactory = factory;
			Tags = tags;

			CreateTabPages();

			UpdateToControls();
		}

		private void CreateTabPages()
		{
			foreach ( IData data in Target.Items )
			{
				var page = new TabPage { Text = data.GetType().ToString() };
				Controller controller = ControllerFactory.Create( data );

				controller.Control.Dock = DockStyle.Fill;

				ControllerCollection.Add( controller );
				page.Controls.Add( controller.Control );

				DataTabControl.TabPages.Add( page );
			}
		}

		private void UpdateToControls()
		{
			NameLabeledTextBox.TextBox.Text = Target.Name;
			TagsListBox.Items.AddRange( Target.Tags.ToArray() );

			ControllerCollection.UpdateToControlAll();
		}

		private void UpdateToTarget()
		{
			Target.Name = NameLabeledTextBox.TextBox.Text;
			Target.Tags.Clear();
			Target.Tags.AddRange( TagsListBox.Items.Cast<string>() );

			ControllerCollection.UpdateToTargetAll();
		}

		private void toolStripButtonRemove_Click( object sender, EventArgs e )
		{
			TagsListBox.RemoveSelectedItems();
		}

		private void toolStripButtonAdd_Click( object sender, EventArgs e )
		{
			using ( var form = new SelectForm { Text = "タグ選択", SelectionMode = SelectionMode.MultiExtended } )
			{
				form.SetSource( Tags );

				if ( form.ShowDialog() ==
				     DialogResult.OK )
				{
					foreach ( object item in form.SelectedItems )
					{
						TagsListBox.BeginUpdate();

						var tag = (string) item;
						if ( !TagsListBox.Items.Contains( tag ) )
						{
							TagsListBox.Items.Add( tag );
						}

						TagsListBox.EndUpdate();
					}
				}
			}
		}
	}
}