﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using ClipExtension.Common;
using ClipExtension.Controls;
using ClipExtension.Main;
using ClipExtension.Options;
using ClipExtension.Plugins;
using Tomham.Windows.Forms;
using Tomham.Windows.Forms.AutoGeneration;

namespace ClipExtension
{
	public partial class OptionsForm : Form
	{
		private readonly ControllerCollection ControllerCollection = new ControllerCollection();
		private ControllerFactory ControllerFactory;
		private CommandTable<GeneralCommandArgs> GeneralCommandTable;

		public OptionsForm()
		{
			InitializeComponent();

			InitializeFontDialog1();
		}

		public AllOptions AllOptions { get; private set; }

		private void InitializeFontDialog1()
		{
			fontDialog1.ShowEffects = false;

			fontDialog1.ShowApply = false;

			fontDialog1.ShowHelp = false;

			fontDialog1.AllowVerticalFonts = false;
		}

		public void SetUp( AllOptions allOptions,
		                   ClipboardManipulator clipboardManipulator,
		                   ControllerFactory controllerFactory,
		                   CommandTable<GeneralCommandArgs> generalCommandTable )
		{
			MaxShowItemCountLabeledNumericUpDown.NumericUpDown.Maximum = 1000;
			MaxShowItemCountLabeledNumericUpDown.NumericUpDown.Increment = 10;
			MaxHistoryItemCountLabeledNumericUpDown.NumericUpDown.Maximum = Repository.MaxHistoryItemCountLimit;
			MaxHistoryItemCountLabeledNumericUpDown.NumericUpDown.Increment = 10;
			MaxHistoryItemCountLabeledNumericUpDown.NumericUpDown.ThousandsSeparator = true;
			CopyReTryDelayLabeledNumericUpDown.NumericUpDown.Maximum = 1000;
			CopyReTryDelayLabeledNumericUpDown.NumericUpDown.Minimum = 50;
			CopyReTryDelayLabeledNumericUpDown.NumericUpDown.Increment = 10;
			CopyReTryTimesLabeledNumericUpDown.NumericUpDown.Maximum = 20;
			BeforePasteDelayLabeledNumericUpDown.NumericUpDown.Maximum = 1000;
			BeforePasteDelayLabeledNumericUpDown.NumericUpDown.Minimum = 50;
			BeforePasteDelayLabeledNumericUpDown.NumericUpDown.Increment = 10;

			CheckAndRegisterChainIntervalLabeledNumericUpDown.NumericUpDown.Maximum = 1000 * 60 * 60;
			CheckAndRegisterChainIntervalLabeledNumericUpDown.NumericUpDown.Increment = 1000 * 10;
			CheckAndRegisterChainIntervalLabeledNumericUpDown.NumericUpDown.ThousandsSeparator = true;
			AllOptions = allOptions;

			ControllerFactory = controllerFactory;
			GeneralCommandTable = generalCommandTable;

			tabControlFormats.Dock = DockStyle.Fill;
			tabControlProcessors.Dock = DockStyle.Fill;

			SetUpAllPluginTabPages( clipboardManipulator );

			UpdateRemainedFromAllOptions();

			ControllerCollection.UpdateToControlAll();
		}

		private void SetUpAllPluginTabPages( ClipboardManipulator clipboardManipulator )
		{
			tabControl1.SuspendLayout();

			SetUpLabeledPluginsPreferenceBox( clipboardManipulator.Processors.Items,
			                                  ProcessorsLabeledPluginsPreferenceBox,
			                                  AllOptions.CommonOptions.ProcessorsPluginsPreference );

			SetUpLabeledPluginsPreferenceBox( clipboardManipulator.FormatSupporterUnit.PluginContainer.Items,
			                                  FormatSupportersLabeledPluginsPreferenceBox,
			                                  AllOptions.CommonOptions.FormatSupportersPluginsPreference );

			SetUpPluginTabPages( clipboardManipulator.Processors.Items, tabControlProcessors );
			SetUpPluginTabPages( clipboardManipulator.FormatSupporterUnit.PluginContainer.Items, tabControlFormats );

			tabControl1.ResumeLayout();
		}

		private void SetUpLabeledPluginsPreferenceBox<T>( IEnumerable<Lazy<T, IPluginMetadata>> plugins,
		                                                  LabeledPluginsPreferenceBox labeledPluginsPreferenceBox,
		                                                  PluginsPreference preference )
		{
			labeledPluginsPreferenceBox.PluginsPreferenceBox.PriorityListBox.Items.AddRange(
				preference.PriorityIDs.ToArray() );

			labeledPluginsPreferenceBox.PluginsPreferenceBox.DefaultListBox.Items.AddRange( CreateDefaultItems( plugins,
			                                                                                                    preference ) );

			labeledPluginsPreferenceBox.TitleText = "動作順序（再起動後に有効）";
			labeledPluginsPreferenceBox.Dock = DockStyle.Fill;
		}

		private object[] CreateDefaultItems<T>( IEnumerable<Lazy<T, IPluginMetadata>> plugins, PluginsPreference preference )
		{
			List<PluginID> allIDs = CreatePluginIDList( plugins );

			IEnumerable<PluginID> removedPriorityItems = allIDs.Except( preference.PriorityIDs );
			return removedPriorityItems.ToArray();
		}

		private List<PluginID> CreatePluginIDList<T>( IEnumerable<Lazy<T, IPluginMetadata>> plugins )
		{
			var ids = new List<PluginID>();
			foreach ( var plugin in plugins )
			{
				ids.Add( PluginID.Create( plugin.Metadata ) );
			}

			return ids;
		}

		private void SetUpPluginTabPages<T>( IEnumerable<Lazy<T, IPluginMetadata>> plugins, TabControl parent )
		{
			foreach ( var plugin in plugins )
			{
				TabPage page = CreateTabPage( plugin );

				parent.TabPages.Add( page );
			}
		}

		private TabPage CreateTabPage<T>( Lazy<T, IPluginMetadata> plugin )
		{
			var page = new TabPage( plugin.Metadata.Name );

			Controller controller = ControllerFactory.Create( plugin.Value );

			controller.Control.Dock = DockStyle.Fill;

			ControllerCollection.Add( controller );
			page.Controls.Add( controller.Control );
			return page;
		}

		private void UpdateRemainedFromAllOptions()
		{
			labeledTextBoxFontSample.TextBox.Font = (Font) AllOptions.IncrementalSearchFormOptions.Font.Clone();
			labeledTextBoxFontSample.TextBox.ForeColor = AllOptions.IncrementalSearchFormOptions.ForeColor;
			labeledTextBoxFontSample.TextBox.BackColor = AllOptions.IncrementalSearchFormOptions.BackColor;

			ShowIncrementalSearchFormLabeledCombinationKeysBox.CombinationKeysBox.CombinationKeys =
				AllOptions.CommonOptions.ShowIncrementalSearchFormKey;
			ShowRepositoryEditFormLabeledCombinationKeysBox1.CombinationKeysBox.CombinationKeys =
				AllOptions.CommonOptions.ShowRepositoryEditFormKey;
			ShowGeneralCommandSearchFormLabeledCombinationKeysBox.CombinationKeysBox.CombinationKeys =
				AllOptions.CommonOptions.ShowGeneralCommandSearchFormKey;

			CopyAndPasteLabeledKeysBox.KeysBox.KeyData = AllOptions.IncrementalSearchFormOptions.CopyAndPasteKey;
			CopyLabeledKeysBox.KeysBox.KeyData = AllOptions.IncrementalSearchFormOptions.CopyKey;
			ShowCommandSearchFormLabeledKeysBox.KeysBox.KeyData =
				AllOptions.IncrementalSearchFormOptions.ShowCommandSearchFormKey;

			MaxHistoryItemCountLabeledNumericUpDown.NumericUpDown.Value = AllOptions.CommonOptions.MaxHistoryItemCount;
			MaxShowItemCountLabeledNumericUpDown.NumericUpDown.Value = AllOptions.IncrementalSearchFormOptions.MaxShowItemCount;
			CopyReTryDelayLabeledNumericUpDown.NumericUpDown.Value = AllOptions.CommonOptions.CopyReTryDelay;
			CopyReTryTimesLabeledNumericUpDown.NumericUpDown.Value = AllOptions.CommonOptions.CopyReTryTimes;
			BeforePasteDelayLabeledNumericUpDown.NumericUpDown.Value = AllOptions.CommonOptions.BeforePasteDelay;
			CheckAndRegisterChainEnabledCheckBox.Checked = AllOptions.CommonOptions.CheckAndRegisterChainEnabled;
			CheckAndRegisterChainIntervalLabeledNumericUpDown.NumericUpDown.Value =
				AllOptions.CommonOptions.CheckAndRegisterChainInterval;

			var pasters = new List<Paster>();

			foreach ( Paster paster in AllOptions.CommonOptions.Pasters )
			{
				pasters.Add( paster.Clone() );
			}

			PastersUpDownListBox.ListBox.Items.AddRange( pasters.ToArray() );

			GeneralCommandHotKeysListBox.Items.AddRange( AllOptions.CommonOptions.GeneralCommandKeys.Cast<object>().ToArray() );
		}

		private void UpdateToAllOptions()
		{
			AllOptions.IncrementalSearchFormOptions.Font = (Font) labeledTextBoxFontSample.TextBox.Font.Clone();
			AllOptions.IncrementalSearchFormOptions.ForeColor = labeledTextBoxFontSample.TextBox.ForeColor;
			AllOptions.IncrementalSearchFormOptions.BackColor = labeledTextBoxFontSample.TextBox.BackColor;

			AllOptions.CommonOptions.ShowIncrementalSearchFormKey =
				ShowIncrementalSearchFormLabeledCombinationKeysBox.CombinationKeysBox.CombinationKeys;
			AllOptions.CommonOptions.ShowRepositoryEditFormKey =
				ShowRepositoryEditFormLabeledCombinationKeysBox1.CombinationKeysBox.CombinationKeys;
			AllOptions.CommonOptions.ShowGeneralCommandSearchFormKey =
				ShowGeneralCommandSearchFormLabeledCombinationKeysBox.CombinationKeysBox.CombinationKeys;

			AllOptions.IncrementalSearchFormOptions.CopyAndPasteKey = CopyAndPasteLabeledKeysBox.KeysBox.KeyData;
			AllOptions.IncrementalSearchFormOptions.CopyKey = CopyLabeledKeysBox.KeysBox.KeyData;
			AllOptions.IncrementalSearchFormOptions.ShowCommandSearchFormKey =
				ShowCommandSearchFormLabeledKeysBox.KeysBox.KeyData;

			AllOptions.CommonOptions.MaxHistoryItemCount = (int) MaxHistoryItemCountLabeledNumericUpDown.NumericUpDown.Value;
			AllOptions.IncrementalSearchFormOptions.MaxShowItemCount =
				(int) MaxShowItemCountLabeledNumericUpDown.NumericUpDown.Value;
			AllOptions.CommonOptions.CopyReTryDelay = (int) CopyReTryDelayLabeledNumericUpDown.NumericUpDown.Value;
			AllOptions.CommonOptions.CopyReTryTimes = (int) CopyReTryTimesLabeledNumericUpDown.NumericUpDown.Value;
			AllOptions.CommonOptions.BeforePasteDelay = (int) BeforePasteDelayLabeledNumericUpDown.NumericUpDown.Value;

			AllOptions.CommonOptions.CheckAndRegisterChainEnabled = CheckAndRegisterChainEnabledCheckBox.Checked;
			AllOptions.CommonOptions.CheckAndRegisterChainInterval =
				(int) CheckAndRegisterChainIntervalLabeledNumericUpDown.NumericUpDown.Value;

			AllOptions.CommonOptions.ProcessorsPluginsPreference.PriorityIDs =
				ProcessorsLabeledPluginsPreferenceBox.PluginsPreferenceBox.PriorityListBox.Items.Cast<PluginID>().ToList();

			AllOptions.CommonOptions.FormatSupportersPluginsPreference.PriorityIDs =
				FormatSupportersLabeledPluginsPreferenceBox.PluginsPreferenceBox.PriorityListBox.Items.Cast<PluginID>().ToList();

			AllOptions.CommonOptions.Pasters.Clear();
			foreach ( object item in PastersUpDownListBox.ListBox.Items )
			{
				AllOptions.CommonOptions.Pasters.Add( (Paster) item );
			}

			AllOptions.CommonOptions.GeneralCommandKeys = new Dictionary<string, CombinationKeys>();
			foreach ( KeyValuePair<string, CombinationKeys> item in GeneralCommandHotKeysListBox.Items )
			{
				AllOptions.CommonOptions.GeneralCommandKeys.Add( item.Key, item.Value );
			}
		}

		private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
		{
			ControllerCollection.UpdateToTargetAll();
			UpdateToAllOptions();
			DialogResult = DialogResult.OK;
		}

		private void toolStripButtonRemove_Click( object sender, EventArgs e )
		{
			PastersUpDownListBox.ListBox.RemoveSelectedItems();
		}

		private void toolStripButtonAdd_Click( object sender, EventArgs e )
		{
			using ( var form = new PasterEditForm() )
			{
				var item = new Paster();

				form.SetUp( item );
				if ( form.ShowDialog() ==
				     DialogResult.OK )
				{
					PastersUpDownListBox.ListBox.Items.Add( item );
				}
			}
		}

		private void toolStripButtonEdit_Click( object sender, EventArgs e )
		{
			EditPaster();
		}

		private void EditPaster()
		{
			if ( PastersUpDownListBox.ListBox.SelectedItems.Count != 1 )
			{
				return;
			}

			using ( var form = new PasterEditForm { StartPosition = FormStartPosition.CenterParent } )
			{
				var item = (Paster) PastersUpDownListBox.ListBox.SelectedItem;

				form.SetUp( item );
				if ( form.ShowDialog() ==
				     DialogResult.OK )
				{
					// 表示の更新のため
					PastersUpDownListBox.ListBox.ReplaceSelectedItem( item );
				}
			}
		}

		private void toolStripButtonCopy_Click( object sender, EventArgs e )
		{
			if ( PastersUpDownListBox.ListBox.SelectedItems.Count == 1 )
			{
				Paster item = ( (Paster) PastersUpDownListBox.ListBox.SelectedItem ).Clone();
				PastersUpDownListBox.ListBox.Items.Add( item );
			}
		}

		private void PastersUpDownListBox_ListBox_DoubleClick( object sender, EventArgs e )
		{
			EditPaster();
		}

		private void GeneralCommandHotKeysListBox_DoubleClick( object sender, EventArgs e )
		{
			EditGeneralCommandHotKey();
		}

		private void toolStripButtonEditGeneralCommandHotKey_Click( object sender, EventArgs e )
		{
			EditGeneralCommandHotKey();
		}

		private void toolStripButtonRemoveGeneralCommandHotKeys_Click( object sender, EventArgs e )
		{
			GeneralCommandHotKeysListBox.RemoveSelectedItems();
		}

		private void EditGeneralCommandHotKey()
		{
			if ( GeneralCommandHotKeysListBox.SelectedItems.Count == 1 )
			{
				var pair = (KeyValuePair<string, CombinationKeys>) GeneralCommandHotKeysListBox.SelectedItem;

				using (
					var form = new CombinationKeysEditForm
					           { StartPosition = FormStartPosition.CenterParent, Text = pair.Key, CombinationKeys = pair.Value } )
				{
					if ( form.ShowDialog() ==
					     DialogResult.OK )
					{
						CombinationKeys keys = form.CombinationKeys;

						if ( keys.KeyCode ==
						     Keys.None )
						{
							keys = new CombinationKeys();
						}

						var item = new KeyValuePair<string, CombinationKeys>( pair.Key, keys );
						GeneralCommandHotKeysListBox.ReplaceSelectedItem( item );
					}
				}
			}
		}

		private void toolStripButtonAddGeneralCommandHotKey_Click( object sender, EventArgs e )
		{
			using ( var form = new SelectForm { Text = "コマンド", SelectionMode = SelectionMode.MultiExtended } )
			{
				form.SetSource( GeneralCommandTable.Commands );

				if ( form.ShowDialog() ==
				     DialogResult.OK )
				{
					foreach ( Command<GeneralCommandArgs> command in form.SelectedItems )
					{
						if ( !ContainsSameNameItemFromGeneralCommandHotKeysListBox( command.Name ) )
						{
							var item = new KeyValuePair<string, CombinationKeys>( command.Name, new CombinationKeys() );
							GeneralCommandHotKeysListBox.Items.Add( item );
						}
					}
				}
			}
		}

		private bool ContainsSameNameItemFromGeneralCommandHotKeysListBox( string name )
		{
			return GeneralCommandHotKeysListBox.Items.Cast<KeyValuePair<string, CombinationKeys>>().Any( item => item.Key == name );
		}

		#region View options events

		private void buttonFont_Click( object sender, EventArgs e )
		{
			fontDialog1.Font = (Font) labeledTextBoxFontSample.TextBox.Font.Clone();
			if ( fontDialog1.ShowDialog() ==
			     DialogResult.OK )
			{
				labeledTextBoxFontSample.TextBox.Font = (Font) fontDialog1.Font.Clone();
			}
		}

		private void buttonTextForeColor_Click( object sender, EventArgs e )
		{
			labeledTextBoxFontSample.TextBox.ForeColor =
				GetColorWithDialog( labeledTextBoxFontSample.TextBox.ForeColor );
		}

		private void buttonTextBackColor_Click( object sender, EventArgs e )
		{
			labeledTextBoxFontSample.TextBox.BackColor =
				GetColorWithDialog( labeledTextBoxFontSample.TextBox.BackColor );
		}

		private Color GetColorWithDialog( Color initialColor )
		{
			using ( var dialog = new ColorDialog { Color = initialColor } )
			{
				if ( dialog.ShowDialog() ==
				     DialogResult.OK )
				{
					return dialog.Color;
				}
			}

			return initialColor;
		}

		#endregion
	}
}