﻿using System;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using Tomham.Collections.Specialized;

namespace ClipExtension
{
	internal static class Program
	{
		private static Mutex Mutex;

		[STAThread]
		private static void Main( string[] args )
		{
			try
			{
				if ( MultipleExecutionMode( args ) )
				{
					Work();
				}
				else
				{
					OnlyOneWork();
				}
			}
			catch ( Exception e )
			{
				string fileName = Path.Combine( Application.StartupPath, "Error.txt" );
				string message = string.Format( "致命的なエラーが発生したので終了します。 {0}に詳細を出力します", fileName );

				var list = new StringList();
				list.AddRange( new[] { e.Message, e.StackTrace } );
				list.SaveToFile( fileName );

				MessageBox.Show( message );
			}
		}

		private static void OnlyOneWork()
		{
			Mutex = new Mutex( false, "ClipExtension" );

			if ( Mutex.WaitOne( 0, false ) )
			{
				try
				{
					Work();
				}
				finally
				{
					Mutex.ReleaseMutex();
				}
			}
		}

		private static void Work()
		{
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault( false );

			var form = new MainForm();

			Application.Run();
		}

		private static bool MultipleExecutionMode( string[] args )
		{
			if ( args.Length == 0 )
			{
				return false;
			}

			return args[0] == "-m";
		}
	}
}