using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections.Specialized;

namespace Tomham.Collections.Specialized
{
	public class StringList : List<string>
	{
		public StringList()
		{
		}

		public StringList( string fileName )
		{
			this.LoadFromFile( fileName );
		}

		public StringList( string fileName, Encoding encoding )
		{
			this.LoadFromFile( fileName, encoding );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="fileName"></param>
		/// <remarks>Encoding.UTF8gp</remarks>
		public void SaveToFile( string fileName )
		{
			this.SaveToFile( fileName, Encoding.UTF8 );
		}

		public void SaveToFile( string fileName, Encoding encoding )
		{
			if ( fileName == null )
			{
				throw new ArgumentNullException( "fileName" );
			}

			if ( encoding == null )
			{
				throw new ArgumentNullException( "encoding" );
			}

			if ( fileName == string.Empty )
			{
				throw new ArgumentException( "fileName is Empty." );
			}

			using ( StreamWriter writer = new StreamWriter( fileName, false, encoding ) )
			{
				foreach ( string line in this )
				{
					writer.WriteLine( line );
				}
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="fileName"></param>
		/// <remarks>Encoding.UTF8gp</remarks>
		public void LoadFromFile( string fileName )
		{
			this.LoadFromFile( fileName, Encoding.UTF8 );
		}

		public void LoadFromFile( string fileName, Encoding encoding )
		{
			if ( fileName == null )
			{
				throw new ArgumentNullException( "fileName" );
			}

			if ( encoding == null )
			{
				throw new ArgumentNullException( "encoding" );
			}

			if ( fileName == string.Empty )
			{
				throw new ArgumentException( "fileName is Empty." );
			}

			if ( !File.Exists( fileName ) )
			{
				throw new FileNotFoundException( "File Not Found", fileName );
			}

			this.Clear();

			using ( StreamReader reader = new StreamReader( fileName, encoding ) )
			{
				while ( -1 < reader.Peek() )
				{
					this.Add( reader.ReadLine() );
				}
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="value"></param>
		/// <returns></returns>
		/// <remarks>ignoreCase = false</remarks>
		new public bool Contains( string value )
		{
			return this.Contains( value, false );
		}

		public bool Contains( string value, bool ignoreCase )
		{
			foreach ( string currentItem in this )
			{
				if ( currentItem == null )
				{
					if ( value == null )
					{
						return true;
					}
				}
				else if ( ignoreCase )
				{
					if (
						currentItem.Equals( value, StringComparison.CurrentCultureIgnoreCase )
						)
					{
						return true;
					}
				}
				else
				{
					if ( currentItem.Equals( value, StringComparison.CurrentCulture ) )
					{
						return true;
					}
				}
			}

			return false;
		}
	}
}
