﻿using System;
using System.Collections.Generic;
using NUnit.Framework;

namespace Tomham.Collections.Generic.Tests
{
	[TestFixture]
	public class CollectionHelperTest : AssertionHelper
	{
		private List<string> CreateList()
		{
			var result = new List<string>();

			result.AddRange( new[] { "ant", "bird", "cat" } );

			return result;
		}

		[Test]
		public void MoveDownItems()
		{
			List<string> actual = CreateList();

			var expected = new[] { "cat", "ant", "bird" };

			actual.MoveDownItems( 0, 1 );

			Assert.That( actual, Is.EqualTo( expected ) );

			actual.MoveDownItems( 1, 2 );

			Assert.That( actual, Is.EqualTo( expected ) );
		}

		[Test]
		public void MoveItemsToFirst()
		{
			List<string> actual = CreateList();

			var expected = new[] { "bird", "cat", "ant" };

			actual.MoveItemsToFirst( new List<string> { "bird", "cat" } );

			Assert.That( actual, Is.EqualTo( expected ) );

			actual.MoveItemsToFirst( new List<string> { "bird", "cat" } );

			Assert.That( actual, Is.EqualTo( expected ) );
		}

		[Test]
		public void MoveItemsToFirstWithBadArgument()
		{
			List<string> actual = CreateList();

			Assert.Throws<ArgumentException>( () => actual.MoveItemsToFirst( new List<string> { "bird", "NotExist" } ) );
		}

		[Test]
		public void MoveUpItems()
		{
			List<string> actual = CreateList();

			var expected = new[] { "bird", "cat", "ant" };

			actual.MoveUpItems( 1, 2 );

			Assert.That( actual, Is.EqualTo( expected ) );

			actual.MoveUpItems( 0, 1 );

			Assert.That( actual, Is.EqualTo( expected ) );
		}

		[Test]
		public void ReferenceIndexOf()
		{
			var item = new object();

			var list = new List<object> { "dog", item };

			Assert.That( list.ReferenceIndexOf( item ), Is.EqualTo( 1 ) );
			Assert.That( list.ReferenceIndexOf( "NotContainsItem" ), Is.EqualTo( -1 ) );
		}

		[Test]
		public void RemoveDuplicateItems()
		{
			var actual = new List<string> { "ant", "ant", "bird" };

			var expected = new[] { "ant", "bird" };

			actual.RemoveDuplicateItems();

			Assert.That( actual, Is.EqualTo( expected ) );
		}

		[Test]
		public void RemoveDuplicateItems2()
		{
			List<string> actual = CreateList();

			List<string> expected = CreateList();

			actual.RemoveDuplicateItems();

			Assert.That( actual, Is.EqualTo( expected ) );
		}

		[Test]
		public void RemoveDuplicateItems3()
		{
			var actual = new List<string> { "ant", "ant", "ant", "bird", "bird" };

			var expected = new[] { "ant", "bird" };

			actual.RemoveDuplicateItems();

			Assert.That( actual, Is.EqualTo( expected ) );
		}

		[Test]
		public void RemoveExcessItems()
		{
			List<string> actual = CreateList();

			var expected = new[] { "ant", "bird" };

			actual.RemoveExcessItems( 2 );

			Assert.That( actual, Is.EqualTo( expected ) );

			actual.RemoveExcessItems( 2 );

			Assert.That( actual, Is.EqualTo( expected ) );

			actual.RemoveExcessItems( 0 );
			Assert.That( actual.Count, Is.EqualTo( 0 ) );
		}

		[Test]
		public void RemoveRange()
		{
			List<string> actual = CreateList();

			var expected = new List<string> { "ant" };

			CollectionHelper.RemoveRange( actual, 1, 2 );
			Assert.That( actual, Is.EqualTo( expected ) );

			CollectionHelper.RemoveRange( actual, 0, 1 );

			Assert.That( actual.Count, Is.EqualTo( 0 ) );

			CollectionHelper.RemoveRange( actual, 0, 0 );

			Assert.That( actual.Count, Is.EqualTo( 0 ) );

			Assert.Throws<ArgumentException>( () => CollectionHelper.RemoveRange( actual, 0, 1 ) );
		}

		[Test]
		public void RemoveReference()
		{
			var item = new object();
			var time = new DateTime();

			var list = new List<object> { time, item };

			Assert.That( list.RemoveReference( item ) );

			Assert.That( list, Is.EqualTo( new List<object> { time } ) );

			Assert.That( list.RemoveReference( item ), Is.EqualTo( false ) );

			Assert.That( list.RemoveReference( time ), Is.EqualTo( false ) );
			Assert.That( list, Is.EqualTo( new List<object> { time } ) );
		}
	}
}