﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace Tomham.Collections.Generic
{
	public static class EnumerableHelper
	{
		public static bool ContainsReference( this IEnumerable enumerable, object item )
		{
			foreach ( object current in enumerable )
			{
				if ( ReferenceEquals( current, item ) )
				{
					return true;
				}
			}

			return false;
		}

		public static int SequenceGetHashCode( this IEnumerable list )
		{
			int result = 0;

			foreach ( object item in list )
			{
				result = item.GetHashCode() ^ result;
			}

			return result;
		}

		public static IList<T> Extract<T>( IEnumerable<T> enumerable, Predicate<T> isMatched )
		{
			if ( enumerable == null )
			{
				throw new ArgumentNullException( "enumerable" );
			}

			if ( isMatched == null )
			{
				throw new ArgumentNullException( "isMatched" );
			}

			var result = new List<T>();

			foreach ( T item in enumerable )
			{
				if ( isMatched( item ) )
				{
					result.Add( item );
				}
			}

			return result;
		}

		public static List<T> ExtractToList<T>( IEnumerable<T> enumerable, Predicate<T> isMatched )
		{
			IList<T> extracted = Extract( enumerable, isMatched );

			return ToList( extracted );
		}

		public static IList<TResult> ExtractOfType<T, TResult>( IEnumerable<T> enumerable )
			where TResult : T
		{
			if ( enumerable == null )
			{
				throw new ArgumentNullException( "enumerable" );
			}

			var result = new List<TResult>();

			foreach ( T item in enumerable )
			{
				if ( item is TResult )
				{
					result.Add( (TResult) item );
				}
			}

			return result;
		}

		public static List<TResult> ExtractOfTypeToList<T, TResult>( IEnumerable<T> enumerable )
			where TResult : T
		{
			IList<TResult> extracted = ExtractOfType<T, TResult>( enumerable );

			return ToList( extracted );
		}

		public static List<T> ToList<T>( IEnumerable<T> enumerable )
		{
			if ( enumerable == null )
			{
				throw new ArgumentNullException( "enumerable" );
			}

			var result = new List<T>();

			foreach ( T item in enumerable )
			{
				result.Add( item );
			}

			return result;
		}
	}
}