using System;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Tomham.Diagnostics
{
	public static class ProcessHelper
	{
		private const int SW_SHOW = 5;
		private const int SW_RESTORE = 9;

		/// <summary>
		/// FileNameArgument؂蕪ProcessStartInfoɊi[ĕԂ
		/// </summary>
		/// <param name="commandLine"></param>
		/// <returns></returns>
		public static ProcessStartInfo CreateProcessStartInfo( string commandLine )
		{
			if ( commandLine == null )
			{
				throw new ArgumentNullException( "commandLine" );
			}

			if ( commandLine == string.Empty )
			{
				throw new ArgumentException( "commandLine == string.Empty" );
			}

			commandLine = commandLine.Trim();

			var result = new ProcessStartInfo();

			char doubleQuotation = Convert.ToChar( "\"" );

			if ( commandLine[0] == doubleQuotation )
			{
				if ( commandLine.Length <= 2 )
				{
					throw new ArgumentException( "commandLine is Invalid: " + commandLine );
				}

				int n = commandLine.IndexOf( doubleQuotation, 1 );

				result.FileName = commandLine.Substring( 1, n - 1 );

				try
				{
					result.Arguments = commandLine.Substring( n + 2 );
				}
				catch
				{
					result.Arguments = string.Empty;
				}
			}
			else
			{
				char[] separator = { Convert.ToChar( " " ) };
				string[] splitedCommandLine = commandLine.Split( separator );

				result.FileName = splitedCommandLine[0];

				try
				{
					int firstSpaceIndex = commandLine.IndexOf( " " );

					if ( firstSpaceIndex != -1 )
					{
						result.Arguments = commandLine.Substring( firstSpaceIndex + 1 );
					}
				}
				catch
				{
					result.Arguments = string.Empty;
				}
			}

			result.FileName = result.FileName.Trim();
			result.Arguments = result.Arguments.Trim();

			return result;
		}

		[DllImport( "USER32.DLL", CharSet = CharSet.Auto )]
		private static extern int ShowWindow(
			IntPtr hWnd,
			int nCmdShow
			);

		[DllImport( "USER32.DLL", CharSet = CharSet.Auto )]
		private static extern int ShowWindowAsync(
			IntPtr hWnd,
			int nCmdShow
			);

		[DllImport( "USER32.DLL", CharSet = CharSet.Auto )]
		private static extern bool SetForegroundWindow( IntPtr hWnd );

		[DllImport( "user32.dll" )]
		private static extern IntPtr GetForegroundWindow();

		[DllImport( "user32.dll" )]
		private static extern uint GetWindowThreadProcessId( IntPtr hWnd, out int lpdwProcessId );

		[DllImport( "user32.dll" )]
		private static extern bool IsIconic( IntPtr hWnd );

		public static bool IsIconic( Process process )
		{
			return IsIconic( process.MainWindowHandle );
		}

		public static Process GetForegroundWindowProcess()
		{
			IntPtr hWnd = GetForegroundWindow();
			int processid;
			GetWindowThreadProcessId( hWnd, out processid );
			return Process.GetProcessById( processid );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="process"></param>
		/// <remarks>ŏĂEBhE͒ʏ̑傫ɖ߂BŏOɍő剻ĂƂĂB</remarks>
		public static void ActivateWindow( Process process )
		{
			if ( IsIconic( process ) )
			{
				ShowWindowAsync( process.MainWindowHandle, SW_RESTORE );
			}
			else
			{
				ShowWindowAsync( process.MainWindowHandle, SW_SHOW );
			}

			SetForegroundWindow( process.MainWindowHandle );
		}
	}
}