﻿using NUnit.Framework;

namespace Tomham.IO.FileManipulation.Tests
{
	[TestFixture]
	public class NormalTextConditionTest
	{
		#region Setup/Teardown

		[SetUp]
		public void SetUp()
		{
			TestHelper.SetUp();
		}

		#endregion

		[Test]
		public void Create()
		{
			TextCondition condition = new NormalTextCondition();

			Assert.IsTrue( condition.IgnoreCase );
			Assert.IsTrue( condition.OnlyNameMatching );
		}

		[Test]
		public void IgnoreCaseIsFalse()
		{
			TextCondition condition = new NormalTextCondition
			                               {
			                               	Pattern = "file.txt",
			                               	IgnoreCase = false
			                               };

			Assert.IsFalse( condition.IsMatch( TestHelper.GetArguments() ) );
		}

		[Test]
		public void IgnoreCaseIsTrue()
		{
			TextCondition condition = new NormalTextCondition
			                               {
			                               	Pattern = "file.txt",
			                               	IgnoreCase = true
			                               };

			Assert.IsTrue( condition.IsMatch( TestHelper.GetArguments() ) );
		}

		[Test]
		public void OnlyNameMattingIsFalse()
		{
			TextCondition condition = new NormalTextCondition
			                               {
			                               	Pattern = "bin",
			                               	OnlyNameMatching = false
			                               };

			Assert.IsTrue( condition.IsMatch( TestHelper.GetArguments() ) );
		}

		[Test]
		public void OnlyNameMattingIsTrue()
		{
			TextCondition condition = new NormalTextCondition
			                               {
			                               	Pattern = "bin",
			                               	OnlyNameMatching = true
			                               };

			Assert.IsFalse( condition.IsMatch( TestHelper.GetArguments() ) );
		}
	}
}