using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace Tomham.IO
{
	/// <summary>
	/// Path֘AStatic Method񋟂
	/// </summary>
	public static class PathHelper
	{
		public const string DoubleQuotation = "\"";
		public const string SingleQuotation = "'";

		public static string GetName( string path )
		{
			if ( IsDrive( path ) )
			{
				return ExcludeTrailingPathDelimiter( path );
			}

			return Path.GetFileName( path );
		}

		/// <summary>
		/// ɃpX؂肪ΒǉāAłȂΓ͂̂܂ܕԂ
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static string IncludeTrailingPathDelimiter( string path )
		{
			if (
				!path.EndsWith( Path.DirectorySeparatorChar.ToString() )
				)
			{
				path += Path.DirectorySeparatorChar;
			}

			return path;
		}

		/// <summary>
		/// ɃpX؂肪΍폜āAłȂΓ͂̂܂ܕԂ
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static string ExcludeTrailingPathDelimiter( string path )
		{
			if (
				path.EndsWith( Path.DirectorySeparatorChar.ToString() )
				)
			{
				path = path.Remove( path.Length - 1, 1 );
			}

			return path;
		}

		/// <summary>
		/// OɃ_uNH[e[VtĕԂ
		/// </summary>
		/// <param name="input"></param>
		/// <returns></returns>
		public static string DoubleQuotString( string input )
		{
			StringBuilder result = new StringBuilder();
			result.Append( DoubleQuotation );
			result.Append( input );
			result.Append( DoubleQuotation );

			return result.ToString();
		}

		/// <summary>
		/// path1؂L𖖔ɎȂDrivełv
		/// </summary>
		/// <param name="path1"></param>
		/// <param name="path2"></param>
		/// <returns></returns>
		public static string Combine( string path1, string path2 )
		{
			if ( IsDrive( path1 ) )
			{
				path1 = IncludeTrailingPathDelimiter( path1 );
			}

			return Path.Combine( path1, path2 );
		}

		public static bool IsDrive( string path )
		{
			Regex regex = new Regex( @"^[a-z]:(\\)?$", RegexOptions.IgnoreCase );

			return regex.IsMatch( path );
		}

		public static string GetDrive( string fullPath )
		{
			if ( fullPath.Length < 2 )
			{
				throw new ArgumentException( "fullPath.Length < 2" );
			}

			string result = fullPath.Substring( 0, 2 );

			if ( IsDrive( result ) )
			{
				return result;
			}

			throw new ArgumentException( "fullPath started without drive path pattern." );
		}

		public static string GetParent( string fullPath )
		{
			if ( IsDrive( fullPath ) )
			{
				return string.Empty;
			}

			fullPath = ExcludeTrailingPathDelimiter( fullPath );

			int length = fullPath.LastIndexOf( Path.DirectorySeparatorChar.ToString() );

			if ( length == 0 )
			{
				throw new ArgumentException();
			}

			return fullPath.Substring( 0, length );
		}

		public static IList<string> GetUnExistsDirectories( string directoryFullPath )
		{
			string drive = GetDrive( directoryFullPath );

			if ( !Directory.Exists( drive ) )
			{
				throw new DriveNotFoundException( string.Format( "{0} was not found.", drive ) );
			}

			directoryFullPath = ExcludeTrailingPathDelimiter( directoryFullPath );

			IList<string> unExistsDirectories = GetUnExistsAncestry( directoryFullPath );

			if ( !Directory.Exists( directoryFullPath ) &&
			     !IsDrive( directoryFullPath ) )
			{
				unExistsDirectories.Add( directoryFullPath );
			}

			return unExistsDirectories;
		}

		private static IList<string> GetUnExistsAncestry( string directory )
		{
			List<string> result = new List<string>();

			PushUnExistsAncestry( directory, result );

			return result;
		}

		private static void PushUnExistsAncestry( string directory, IList<string> result )
		{
			if ( IsDrive( directory ) )
			{
				return;
			}

			string parent = GetParent( directory );

			if ( !Directory.Exists( parent ) )
			{
				result.Insert( 0, parent );
				PushUnExistsAncestry( parent, result );
			}
		}
	}
}