﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Tomham.IO
{
	public abstract class Serializer<T>
	{
		private IEqualityComparer<T> EqualityComparerStore;
		public ReWriteMode ReWriteMode { get; set; }

		public IEqualityComparer<T> EqualityComparer
		{
			get
			{
				if ( EqualityComparerStore == null )
				{
					EqualityComparerStore = EqualityComparer<T>.Default;
				}

				return EqualityComparerStore;
			}
			set { EqualityComparerStore = value; }
		}

		public bool ForceSerialization { get; set; }

		public void Serialize( T data, string fileName )
		{
			if ( data == null )
			{
				throw new ArgumentNullException( "data" );
			}

			switch ( ReWriteMode )
			{
				case ReWriteMode.NotEqual:

					if ( !File.Exists( fileName ) )
					{
						DoSerialize( data, fileName );
						break;
					}

					if ( !EqualsOld( data, fileName ) )
					{
						DoSerialize( data, fileName );
					}

					break;

				case ReWriteMode.NotExist:

					if ( !File.Exists( fileName ) )
					{
						DoSerialize( data, fileName );
					}

					break;

				case ReWriteMode.All:

					DoSerialize( data, fileName );
					break;

				default:
					throw new ArgumentOutOfRangeException();
			}
		}

		private bool EqualsOld( T data, string fileName )
		{
			T old;

			try
			{
				old = DeSerialize( fileName );
			}
			catch
			{
				if ( ForceSerialization )
				{
					return false;
				}

				throw;
			}

			return EqualityComparer.Equals( data, old );
		}

		protected abstract void DoSerialize( T data, string fileName );

		public abstract T DeSerialize( string fileName );
	}
}