﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Tomham.Windows.Forms;

namespace ActiveTest
{
	public partial class Form1 : Form
	{
		private readonly SampleController Controller = new SampleController( new Class1() ) { Margin = 0 };

		private readonly WindowStates WindowStates = new WindowStates
		                                             {
		                                             	Size = new Size( 100, 100 ),
		                                             	StartPosition = FormStartPosition.CenterParent
		                                             };

		private readonly HotKey hot1 = new HotKey();
		private readonly HotKey hot2 = new HotKey();

		public Form1()
		{
			InitializeComponent();
		}

		private void Form1_Load( object sender, EventArgs e )
		{
			hot1.Pressed += hot1_Pressed;
			hot2.Pressed += hot2_Pressed;
			Controls.Add( Controller.Control );
			Controller.Control.Location = new Point( 500, 500 );
			Test();

			Text += Keys.Alt.ToString();
			Text += ( Keys.Control | Keys.Shift | Keys.A ).ToString();

			var a = new CombinationKeys { KeyCode = ( Keys.A ), Modifiers = Modifiers.Control | Modifiers.Shift };
			Text += a.ToString();
			var b = new CombinationKeys { Modifiers = Modifiers.Alt };
			Text += b;
		}

		private void Test()
		{
			listBox1.SelectionMode = SelectionMode.MultiExtended;
			labeledUpDownListBox1.UpDownListBox.ListBox.SelectionMode = SelectionMode.MultiExtended;
			var list = new List<string>();
			for ( int i = 0; i < 25; i++ )
			{
				list.Add( "Item" + i.ToString() );

				comboBox1.Items.Add( "test" + i.ToString() );
			}
			comboBox1.AutoCompleteSource = AutoCompleteSource.ListItems;

			labeledUpDownListBox1.UpDownListBox.ListBox.Items.AddRange( list.ToArray() );

			filterListBox1.SetSource( list );
			filterListBox1.SelectionMode = SelectionMode.MultiExtended;
		}

		private void hot2_Pressed( object sender, EventArgs e )
		{
			Text += "hot2";
		}

		private void hot1_Pressed( object sender, EventArgs e )
		{
			Text += "hot1";
		}

		private void button1_Click( object sender, EventArgs e )
		{
			hot1.CombinationKeys = combinationKeysBox1.CombinationKeys;
			hot2.CombinationKeys = combinationKeysBox2.CombinationKeys;
			checkBox1.Checked = hot1.HasRegistered;
			checkBox2.Checked = hot2.HasRegistered;
		}

		private void Form1_FormClosed( object sender, FormClosedEventArgs e )
		{
			hot1.Dispose();
			hot2.Dispose();
		}

		private void button2_Click( object sender, EventArgs e )
		{
			using ( var form = new SelectForm() )
			{
				form.Text = "test";
				var items = new List<string> { "a1", "b2", "a3" };
				form.SetSource( items );
				form.SelectionMode = SelectionMode.MultiExtended;

				if ( form.ShowDialog() ==
				     DialogResult.OK )
				{
					foreach ( object item in form.SelectedItems )
					{
						Text += item.ToString();
					}
				}
			}
		}

		private void button3_Click( object sender, EventArgs e )
		{
			using ( var form = new SelectForm() )
			{
				form.SetSource( new List<string> { "a" } );

				form.ShowDialog();

				if ( form.SelectedItem == null )
				{
					Text = "null";
				}
				else
				{
					Text = form.SelectedItem.ToString();
				}
			}
		}

		private void button4_Click( object sender, EventArgs e )
		{
			using ( var form = new SelectForm() )
			{
				form.SetSource( new List<string> { "a" } );
				form.WindowState = FormWindowState.Maximized;
				WindowStates.UpdateToForm( form );
				form.ShowDialog();
				WindowStates.UpdateFromForm( form );
			}
		}

		private void comboBox1_TextChanged( object sender, EventArgs e )
		{
			Text = comboBox1.Text;
		}

		private void filterListBox1_ComboBoxKeyDown( object sender, KeyEventArgs e )
		{
			Text = e.KeyData.ToString();
		}
	}
}