﻿using System.Windows.Forms;
using NUnit.Framework;

namespace Tomham.Windows.Forms.Tests
{
	[TestFixture]
	public class KeysHelperTest
	{
		[Test]
		public void IsModifiersOnlyWithCombinationKeys()
		{
			Assert.That( !KeysHelper.IsModifiersOnly( new CombinationKeys() ) );

			var onlyA = new CombinationKeys { KeyCode = Keys.A };
			Assert.That( !KeysHelper.IsModifiersOnly( onlyA ) );

			var altA = new CombinationKeys { KeyCode = Keys.A, Modifiers = Modifiers.Alt };
			Assert.That( !KeysHelper.IsModifiersOnly( altA ) );

			var altOnly = new CombinationKeys { Modifiers = Modifiers.Alt };
			Assert.That( KeysHelper.IsModifiersOnly( altOnly ) );

			var modifiresOnly = new CombinationKeys { Modifiers = Modifiers.Alt | Modifiers.Windows };
			Assert.That( KeysHelper.IsModifiersOnly( modifiresOnly ) );
		}

		[Test]
		public void IsModifiersOnlyWithKeys()
		{
			Assert.That( !KeysHelper.IsModifiersOnly( Keys.None ) );
			Assert.That( !KeysHelper.IsModifiersOnly( Keys.A ) );
			Assert.That( !KeysHelper.IsModifiersOnly( Keys.A | Keys.Alt ) );
			Assert.That( KeysHelper.IsModifiersOnly( Keys.Alt ) );
			Assert.That( KeysHelper.IsModifiersOnly( Keys.Alt | Keys.Control ) );
		}
	}
}