﻿using System.Windows.Forms;

namespace Tomham.Windows.Forms.AutoGeneration
{
	public abstract class AutoAdjustedController<T> : Controller
	{
		protected AutoAdjustedController( object target ) : base( target )
		{
			Margin = 3;
		}

		public int Margin { get; set; }

		protected T CastedTarget
		{
			get { return (T) Target; }
		}

		protected Panel RootPanel { get; private set; }

		protected override Control CreateControl()
		{
			RootPanel = new Panel();

			SetUpContentControls();

			return RootPanel;
		}

		protected abstract void SetUpContentControls();

		protected void AddContent( Control content )
		{
			RootPanel.Controls.Add( content );
			AdjustControls();
		}

		protected void RemoveContent( Control content )
		{
			RootPanel.Controls.Remove( content );
			content.Dispose();
			AdjustControls();
		}

		protected void AdjustControls()
		{
			AdjustControl( RootPanel );
		}

		private void AdjustControl( Control control )
		{
			int top = Margin;

			foreach ( Control child in control.Controls )
			{
				child.Left = Margin;
				child.Top = top;
				top += child.Height + Margin;
			}

			control.Height = top;

			int width = 50;

			foreach ( Control child in control.Controls )
			{
				if ( width < child.Width )
				{
					width = child.Width;
				}
			}

			control.Width = width + Margin;
		}
	}
}