﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Tomham.Windows.Forms.AutoGeneration
{
	public class CompositeControllerFactory : ControllerFactory
	{
		private readonly List<ControllerFactory> Items = new List<ControllerFactory>();

		public override bool IsTarget( object target )
		{
			return Items.Any( item => item.IsTarget( target ) );
		}

		protected override Controller DoCreate( object target )
		{
			foreach ( ControllerFactory item in Items )
			{
				if ( item.IsTarget( target ) )
				{
					return item.Create( target );
				}
			}

			throw new InvalidOperationException( "target is not supported." );
		}

		public void Add( ControllerFactory controllerFactory )
		{
			Items.Add( controllerFactory );
		}
	}
}