﻿using System;
using System.Windows.Forms;

namespace Tomham.Windows.Forms.AutoGeneration
{
	/// <summary>
	/// Targetとその編集用、もしくは表示用Controlを分離し、間を取り持つ
	/// </summary>
	public abstract class Controller
	{
		private Control ControlStore;

		protected Controller( object target )
		{
			if ( target == null )
			{
				throw new ArgumentNullException( "target" );
			}

			Target = target;
		}

		public object Target { get; private set; }

		public Control Control
		{
			get
			{
				if ( ControlStore == null )
				{
					ControlStore = CreateControl();
					UpdateToControl();
				}

				return ControlStore;
			}
			set { ControlStore = value; }
		}

		protected abstract Control CreateControl();

		public abstract void UpdateToTarget();
		public abstract void UpdateToControl();
	}
}