﻿using System;
using System.Reflection;

namespace Tomham.Windows.Forms.AutoGeneration
{
	public class TypeRegisterControllerFactory : ControllerFactory
	{
		private readonly Type ControllerTypeStroe;
		private readonly Type TargetTypeStroe;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="targetType"></param>
		/// <param name="controllerType">Need matched constructor.</param>
		public TypeRegisterControllerFactory( Type targetType, Type controllerType )
		{
			if ( !HasMatchedConstructor( controllerType ) )
			{
				throw new ArgumentException( "controllerType has not matched constructor." );
			}

			TargetTypeStroe = targetType;
			ControllerTypeStroe = controllerType;
		}

		public Type TargetType
		{
			get { return TargetTypeStroe; }
		}

		public Type ControllerType
		{
			get { return ControllerTypeStroe; }
		}

		public override bool IsTarget( object target )
		{
			return target.GetType() == TargetType;
		}

		protected override Controller DoCreate( object target )
		{
			ConstructorInfo info = ControllerType.GetConstructor( new[] { typeof ( object ) } );

			Controller result = (Controller) info.Invoke( new[] { target } );

			return result;
		}

		private bool HasMatchedConstructor( Type controller )
		{
			ConstructorInfo info = controller.GetConstructor( new[] { typeof ( object ) } );

			return info != null;
		}
	}
}