﻿using System;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
	[Serializable]
	public struct CombinationKeys
	{
		private Keys KeyCodeStore;
		public Modifiers Modifiers { get; set; }

		public Keys KeyCode
		{
			get { return KeyCodeStore; }
			set { SetKeyCode( value ); }
		}

		private void SetKeyCode( Keys value )
		{
			KeyCodeStore = KeysHelper.ExtractKeyCode( value );
		}

		public override bool Equals( object obj )
		{
			if ( ReferenceEquals( null, obj ) )
			{
				return false;
			}
			if ( obj.GetType() !=
			     typeof ( CombinationKeys ) )
			{
				return false;
			}

			var other = (CombinationKeys) obj;

			if ( !KeyCode.Equals( other.KeyCode ) )
			{
				return false;
			}

			if ( !Modifiers.Equals( other.Modifiers ) )
			{
				return false;
			}

			return true;
		}

		public override int GetHashCode()
		{
			unchecked
			{
				return ( KeyCode.GetHashCode() * 397 ) ^ Modifiers.GetHashCode();
			}
		}

		public override string ToString()
		{
			if ( Modifiers == Modifiers.None )
			{
				return KeyCode.ToString();
			}

			if ( KeyCode == Keys.None )
			{
				return Modifiers.ToString();
			}

			return string.Format( "{0} + {1}", Modifiers.ToString(), KeyCode.ToString() );
		}
	}
}