﻿using System.ComponentModel;
using System.Linq;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
	public partial class CombinationKeysBox : UserControl
	{
		public CombinationKeysBox()
		{
			InitializeComponent();
			InitializeKeyCodeComboBox();
		}

		[Browsable( false )]
		public bool Alt
		{
			get { return AltCheckBox.Checked; }
			set { AltCheckBox.Checked = value; }
		}

		[Browsable( false )]
		public bool Control
		{
			get { return ControlCheckBox.Checked; }
			set { ControlCheckBox.Checked = value; }
		}

		[Browsable( false )]
		public bool Shift
		{
			get { return ShiftCheckBox.Checked; }
			set { ShiftCheckBox.Checked = value; }
		}

		[Browsable( false )]
		public bool Windows
		{
			get { return WindowsCheckBox.Checked; }
			set { WindowsCheckBox.Checked = value; }
		}

		[Browsable( false )]
		public CombinationKeys CombinationKeys
		{
			get
			{
				Keys keyCode = GetKeyCode();
				Modifiers modifiers = GetModifiers();
				return new CombinationKeys { KeyCode = keyCode, Modifiers = modifiers };
			}
			set
			{
				SetKeyCode( value.KeyCode );
				SetModifiers( value.Modifiers );
			}
		}

		private void SetKeyCode( Keys keys )
		{
			if ( KeyCodeComboBox.Items.Contains( keys ) )
			{
				KeyCodeComboBox.SelectedItem = keys;
			}
			else
			{
				KeyCodeComboBox.SelectedIndex = -1;
			}
		}

		private Keys GetKeyCode()
		{
			if ( 0 <= KeyCodeComboBox.SelectedIndex )
			{
				return (Keys) KeyCodeComboBox.SelectedItem;
			}
			else
			{
				return Keys.None;
			}
		}

		private void SetModifiers( Modifiers modifiers )
		{
			AltCheckBox.Checked = KeysHelper.ContainsAlt( modifiers );
			ControlCheckBox.Checked = KeysHelper.ContainsControl( modifiers );
			ShiftCheckBox.Checked = KeysHelper.ContainsShift( modifiers );
			WindowsCheckBox.Checked = KeysHelper.ContainsWindows( modifiers );
		}

		private Modifiers GetModifiers()
		{
			Modifiers result = Modifiers.None;

			if ( AltCheckBox.Checked )
			{
				result |= Modifiers.Alt;
			}
			if ( ControlCheckBox.Checked )
			{
				result |= Modifiers.Control;
			}
			if ( ShiftCheckBox.Checked )
			{
				result |= Modifiers.Shift;
			}
			if ( WindowsCheckBox.Checked )
			{
				result |= Modifiers.Windows;
			}

			return result;
		}

		private void InitializeKeyCodeComboBox()
		{
			KeyCodeComboBox.Items.AddRange( KeysHelper.GetKeyCodes().Cast<object>().ToArray() );
		}
	}
}