﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
	public static class InputKeys
	{
		[DllImport( "user32.dll" )]
		private static extern uint keybd_event( byte bVk, byte bScan, uint dwFlags, UIntPtr dwExtraInfo );

		public static void Input( Keys keys )
		{
			List<byte> modifiers = CreateModifiers( keys );
			byte keyCode = Convert.ToByte( ( KeysHelper.ExtractKeyCode( keys ) ) );

			KeyDown( modifiers, keyCode );

			KeyUp( modifiers, keyCode );
		}

		private static List<byte> CreateModifiers( Keys keys )
		{
			var modifiers = new List<byte>();

			if ( KeysHelper.ContainsShift( keys ) )
			{
				modifiers.Add( VirtualKey.SHIFT );
			}

			if ( KeysHelper.ContainsControl( keys ) )
			{
				modifiers.Add( VirtualKey.CONTROL );
			}

			if ( KeysHelper.ContainsAlt( keys ) )
			{
				modifiers.Add( VirtualKey.MENU );
			}

			return modifiers;
		}

		private static void KeyDown( IEnumerable<byte> modifiers, byte keyCode )
		{
			foreach ( byte modifier in modifiers )
			{
				keybd_event( modifier, 0, 0, (UIntPtr) 0 );
			}

			if ( KeyCodeEnabled( keyCode ) )
			{
				keybd_event( keyCode, 0, 0, (UIntPtr) 0 );
			}
		}

		private static void KeyUp( IEnumerable<byte> modifiers, byte keyCode )
		{
			if ( KeyCodeEnabled( keyCode ) )
			{
				keybd_event( keyCode, 0, 2, (UIntPtr) 0 );
			}

			foreach ( byte modifier in modifiers )
			{
				keybd_event( modifier, 0, 2, (UIntPtr) 0 );
			}
		}

		private static bool KeyCodeEnabled( byte keyCode )
		{
			return 0 < keyCode;
		}
	}
}