﻿using System.Linq;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
	public partial class KeysBox : UserControl
	{
		public KeysBox()
		{
			InitializeComponent();
			InitializeKeyCodeComboBox();
		}

		public bool Alt
		{
			get { return AltCheckBox.Checked; }
			set { AltCheckBox.Checked = value; }
		}

		public bool Control
		{
			get { return ControlCheckBox.Checked; }
			set { ControlCheckBox.Checked = value; }
		}

		public bool Shift
		{
			get { return ShiftCheckBox.Checked; }
			set { ShiftCheckBox.Checked = value; }
		}

		public Keys KeyCode
		{
			get { return GetKeyCode(); }
			set { SetKeyCode( value ); }
		}

		public Keys KeyData
		{
			get
			{
				Keys result = KeyCode;

				if ( AltCheckBox.Checked )
				{
					result |= Keys.Alt;
				}
				if ( ControlCheckBox.Checked )
				{
					result |= Keys.Control;
				}
				if ( ShiftCheckBox.Checked )
				{
					result |= Keys.Shift;
				}

				return result;
			}
			set
			{
				Alt = KeysHelper.ContainsAlt( value );
				Control = KeysHelper.ContainsControl( value );
				Shift = KeysHelper.ContainsShift( value );

				KeyCode = KeysHelper.ExtractKeyCode( value );
			}
		}

		private void InitializeKeyCodeComboBox()
		{
			KeyCodeComboBox.Items.AddRange( KeysHelper.GetKeyCodes().Cast<object>().ToArray() );
		}

		private void SetKeyCode( Keys keys )
		{
			if ( KeyCodeComboBox.Items.Contains( keys ) )
			{
				KeyCodeComboBox.SelectedItem = keys;
			}
			else
			{
				KeyCodeComboBox.SelectedIndex = -1;
			}
		}

		private Keys GetKeyCode()
		{
			if ( 0 <= KeyCodeComboBox.SelectedIndex )
			{
				return (Keys) KeyCodeComboBox.SelectedItem;
			}
			else
			{
				return Keys.None;
			}
		}
	}
}