﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
	/// <summary>
	/// StartPosition等をそれなりに保存・復元する。出来が悪い。
	/// </summary>
	/// <remarks>form.StartPosition == WindowsDefaultBounds or WindowsDefaultLocationの時の事は考えていない</remarks>
	[Serializable]
	public class WindowStates
	{
		public WindowStates()
		{
			StartPosition = FormStartPosition.CenterParent;
		}

		public FormWindowState WindowState { get; set; }
		public FormStartPosition StartPosition { get; set; }
		public Size Size { get; set; }
		public Point DesktopLocation { get; set; }

		public void UpdateToForm( Form form )
		{
			form.StartPosition = StartPosition;
			form.Size = Size;

			if ( StartPosition == FormStartPosition.Manual )
			{
				form.DesktopLocation = DesktopLocation;
			}

			form.WindowState = WindowState;
		}

		public void UpdateFromForm( Form form )
		{
			StartPosition = form.StartPosition;
			WindowState = form.WindowState;

			if ( WindowState == FormWindowState.Normal )
			{
				Size = form.Size;

				if ( StartPosition == FormStartPosition.Manual )
				{
					DesktopLocation = form.DesktopLocation;
				}
			}
		}

		public bool Equals( WindowStates other )
		{
			if ( ReferenceEquals( null, other ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, other ) )
			{
				return true;
			}
			return Equals( other.WindowState, WindowState ) && other.Size.Equals( Size ) &&
			       other.StartPosition.Equals( StartPosition );
		}

		public override bool Equals( object obj )
		{
			if ( ReferenceEquals( null, obj ) )
			{
				return false;
			}
			if ( ReferenceEquals( this, obj ) )
			{
				return true;
			}
			if ( obj.GetType() !=
			     typeof ( WindowStates ) )
			{
				return false;
			}
			return Equals( (WindowStates) obj );
		}

		public override int GetHashCode()
		{
			unchecked
			{
				int result = WindowState.GetHashCode();
				result = ( result * 397 ) ^ Size.GetHashCode();
				result = ( result * 397 ) ^ StartPosition.GetHashCode();
				return result;
			}
		}
	}
}